<template>
  <div class="login-bg">
    <img src="@/assets/imgs/login-name.png" alt="一体化运维服务支撑平台" class="logo-name"/>
    <div class="index_from">
      <div class="page-account-top">
        欢迎登录
      </div>
      <el-form
        ref="loginForm"
        :model="loginForm"
        :rules="loginRules"
        class="login-form"
        autocomplete="on"
        label-position="left"
        @keyup.enter="handleLogin"
      >
        <el-form-item prop="account">
          <el-input
            v-model.trim="loginForm.account"
            prefix-icon="el-icon-user"
            placeholder="用户名"
            type="text"
            tabindex="1"
            autocomplete="off"
          />
        </el-form-item>

        <el-form-item prop="password">
          <el-input
            :key="passwordType"
            v-model.trim="loginForm.password"
            prefix-icon="el-icon-lock"
            :type="passwordType"
            placeholder="密码"
            tabindex="2"
            auto-complete="off"
          />
          <span class="show-pwd" @click="showPwd">
              <svg-icon :icon-class="passwordType === 'password' ? 'eye' : 'eye-open'"/>
            </span>
        </el-form-item>

        <el-form-item prop="code" class="captcha" v-if="loginForm.verifyType==='code' && captchaEnabled">
          <div class="captcha">
            <el-input
              v-model.trim="loginForm.code"
              style="width: 218px"
              prefix-icon="el-icon-message"
              placeholder="验证码"
              type="text"
              tabindex="3"
              autocomplete="off"
            />
            <div class="imgs" @click="getCode()">
              <!--                <img :src="captchatImg" />-->
              <img :src="codeUrl"/>
              <span v-show="showCaptchatImg">已失效</span>
            </div>
          </div>
        </el-form-item>

        <div class="acea-row">
          <el-button
            :loading="loading"
            type="primary"
            style="width: 100%; margin-bottom: 80px"
            @click.native.prevent="handleLogin"
            :disabled="disabled"
          >登录
          </el-button>
        </div>
      </el-form>

    </div>
  </div>
</template>

<script>
import {validUsername} from '@/utils/validate';
import {getCodeImg, getCodeNone} from '@/api/user';

export default {
  name: 'Login',
  data() {
    const validateUsername = (rule, value, callback) => {
      if (!validUsername(value)) {
        callback(new Error('Please enter the correct user name'));
      } else {
        callback();
      }
    };
    const validatePassword = (rule, value, callback) => {
      if (value.length < 6 || value.length > 12) {
        callback(new Error('密码位数为6-12位'));
      } else {
        callback();
      }
    };
    return {
      verify: false,  // 滑动校验
      showCaptchatImg: false,
      // captchatImg: '',
      codeUrl: "",
      fullWidth: document.body.clientWidth,
      swiperOption: {
        pagination: {
          el: '.pagination',
        },
        autoplay: {
          enabled: false,
          disableOnInteraction: false,
          delay: 3000,
        },
      },
      loginForm: {
        account: 'supervisor', // admin
        password: '123456',
        // key: '',
        uuid: '',
        code: '',
        loginType: 'user_pass',  // 用户名、密码方式登录
        verifyType: 'code'     //  'code': 验证码  'slide':滑块  'jigsaw':拼图
        // captcha: {
        //   captchaVerification: '',
        //   secretKey: '',
        //   token: '',
        // },
      },
      loginRules: {
        account: [{required: true, trigger: 'blur', message: '请输入用户名'}], // validator: validateUsername
        password: [{required: true, trigger: 'blur', message: '请输入密码'}],
        code: [{required: true, message: '请输入正确的验证码', trigger: 'blur'}],
      },
      passwordType: 'password',
      capsTooltip: false,
      loading: false,
      showDialog: false,
      redirect: undefined,
      otherQuery: {},
      disabled: false,
      isWeixin: this.$wechat.isWeixin(),
      // 验证码开关
      captchaEnabled: true
    };
  },
  components: {},
  watch: {
    fullWidth(val) {
      // 为了避免频繁触发resize函数导致页面卡顿，使用定时器
      if (!this.timer) {
        // 一旦监听到的screenWidth值改变，就将其重新赋给data里的screenWidth
        this.screenWidth = val;
        this.timer = true;
        const that = this;
        setTimeout(function () {
          // 打印screenWidth变化的值
          that.timer = false;
        }, 400);
      }
    },
    $route: {
      handler: function (route) {
        const query = route.query;
        if (query) {
          this.redirect = query.redirect;
          this.otherQuery = this.getOtherQuery(query);
        }
      },
      immediate: true,
    },
  },
  created() {
    const _this = this;
    document.onkeydown = function (e) {
      if (_this.$route.path.indexOf('login') !== -1) {
        const key = window.event.keyCode;
        if (key === 13) {
          _this.handleLogin();
        }
      }
    };
    window.addEventListener('resize', this.handleResize);
  },
  mounted() {
    if (this.loginForm.account === '') {
      this.$refs.account.focus();
    } else if (this.loginForm.pwd === '') {
      this.$refs.password.focus();
    }
    this.getCode();
  },
  methods: {
    verifyTrue() {
      this.verify = true
    },
    verifyFalse() {
      this.verify = false
    },

    checkCapslock(e) {
      const {key} = e;
      this.capsTooltip = key && key.length === 1 && key >= 'A' && key <= 'Z';
    },
    showPwd() {
      if (this.passwordType === 'password') {
        this.passwordType = '';
      } else {
        this.passwordType = 'password';
      }
      this.$nextTick(() => {
        this.$refs.pwd.focus();
      });
    },
    handleLogin() {
      this.$refs.loginForm.validate((valid) => {
        if (valid) {
          this.success(null);
        } else {
          return false;
        }
      });
    },
    success(params) {
      this.loginForm.captcha = this.$store.state.user.captcha;
      const loading = this.$loading({
        lock: true,
        text: '正在登录中.',
      });
      this.$store
        .dispatch('user/login', this.loginForm)
        .then(() => {
          this.$store.dispatch('user/getInfo').then(res => {
            this.$router.push({path: this.redirect || '/dashboard', query: this.otherQuery});
            loading.close();
            this.disabled = true;
          }).catch((err) => {
            loading.close();
            this.disabled = false;
            if (this.$wechat.isPhone()) this.$dialog.error(err.message);
            this.loginForm.code = null;
            this.getCode();
          })
        })
        .catch((err) => {
          loading.close();
          this.disabled = false;
          if (this.$wechat.isPhone()) this.$dialog.error(err.message);
          this.loginForm.code = null;
          this.getCode();
        });
    },

    getCode() {
      if (this.loginForm.verifyType === 'code') {
        getCodeImg().then(data => {
          this.captchaEnabled = data.captchaEnabled === undefined ? true : data.captchaEnabled;
          if (this.captchaEnabled) {
            this.codeUrl = "data:image/jpeg;base64," + data.img;
          }
          // 有没有验证码，都需要uuid作为唯一标识，2022-12-01
          this.loginForm.uuid = data.uuid;

          // 2023-05-12 按新方法，失败会显示验证码无效
          this.showCaptchatImg = false;
          setTimeout(() => {
            this.showCaptchatImg = true;
          }, 270000);

        }).catch(err => {
          console.log(err);
        });

      } else if (this.loginForm.verifyType === 'jigsaw') {
        // 拼图方式，看需要啥参数，暂未实现
        this.loginForm.code = 'jigsaw'; // 填充code默认值，否则验证不通过。2023-04-06
        this.loginForm.uuid = '';       // 清空验证码，为下次重新验证准备。2023-04-16

      } else if (this.loginForm.verifyType === 'slide' || this.loginForm.verifyType === 'sms') {
        // 不需要验证码，但仍需要请求接口获取uuid
        getCodeNone().then(data => {
          this.loginForm.uuid = data.uuid;
        }).catch(err => {
          console.error(err);
        });

      } else {
        this.$message.error('不支持的验证码类型:' + this.loginForm.verifyType);
      }
    },
    getOtherQuery(query) {
      return Object.keys(query).reduce((acc, cur) => {
        if (cur !== 'redirect') {
          acc[cur] = query[cur];
        }
        return acc;
      }, {});
    }
  }
};
</script>

<style lang="scss" scoped>
$screen-md: 768px;
$font-size-base: 14px;
$animation-time: 0.3s;
$animation-time-quick: 0.15s;
$transition-time: 0.2s;
$ease-in-out: ease-in-out;
$subsidiary-color: #808695;
.login-bg {
  min-width: 1200px;
  margin: 0 auto;
  height: 100%;
  background: url("../../assets/imgs/login-bg.png") center no-repeat;
  background-size: 1920px 1080px;
}

.logo-name {
  position: absolute;
  top: 20px;
  left: 20px;
}

.wechat {
  width: 26px;
  height: 26px;

  img {
    width: 100%;
    height: 100%;
  }
}

.page-account {
  display: flex;
  flex-direction: column;
  justify-content: center;
  align-items: center;
  height: 100vh;
  overflow: auto;

  &-container {
    flex: 1;
    padding: 32px 0;
    text-align: center;
    width: 384px;
    margin: 0 auto;

    &-result {
      width: 100%;
    }
  }

  &-tabs {
    .ivu-tabs-bar {
      border-bottom: none;
    }

    .ivu-tabs-nav-scroll {
      text-align: center;
    }

    .ivu-tabs-nav {
      display: inline-block;
      float: none;
    }
  }

  &-top {
    padding: 32px 0;

    &-logo {
      img {
        max-height: 75px;
      }
    }

    &-desc {
      font-size: $font-size-base;
      color: $subsidiary-color;
    }
  }

  &-auto-login {
    margin-bottom: 24px;
    text-align: left;

    a {
      float: right;
    }
  }

  &-other {
    margin: 24px 0;
    text-align: left;

    span {
      font-size: $font-size-base;
    }

    img {
      width: 24px;
      margin-left: 16px;
      cursor: pointer;
      vertical-align: middle;
      opacity: 0.7;
      transition: all $transition-time $ease-in-out;

      &:hover {
        opacity: 1;
      }
    }
  }

  .ivu-poptip,
  .ivu-poptip-rel {
    display: block;
  }

  &-register {
    float: right;

    &-tip {
      text-align: left;

      &-title {
        font-size: $font-size-base;
      }

      &-desc {
        white-space: initial;
        font-size: $font-size-base;
        margin-top: 6px;
      }
    }
  }

  &-to-login {
    text-align: center;
    margin-top: 16px;
  }

  &-header {
    text-align: right;
    position: fixed;
    top: 16px;
    right: 24px;
  }
}

@media (min-width: $screen-md) {
  .page-account {
    background-repeat: no-repeat;
    background-position: center;
    background-size: cover;
  }
}

.page-account {
  display: flex;
}

.page-account .code {
  display: flex;
  align-items: center;
  justify-content: center;
}

.page-account .code .pictrue {
  height: 40px;
}

.swiperPross {
  border-radius: 6px 0px 0px 6px;
  overflow: hidden;
}

.swiperPross,
.swiperPic,
.swiperPic img {
  width: 286px;
  height: 100%;
}

.swiperPic img {
  width: 100%;
  height: 100%;
}

.container {
  padding: 0 !important;
  z-index: 1;
  display: flex;
}

.index_from {
  width: 400px;
  position: absolute;
  top: 30%;
  right: 24%;
  margin: 0 auto;
  padding: 30px 50px;
  border-radius: 16px;
  background: #FFFFFF;
}

.page-account-top {
  padding: 20px 0 !important;
  font-size: 18px;
  font-weight: bold;
  text-align: center;
  margin-bottom: 20px;
}

.btn {
  background: linear-gradient(90deg, rgba(25, 180, 241, 1) 0%, rgba(14, 115, 232, 1) 100%) !important;
}
</style>

<style lang="scss" scoped>
.captcha {
  display: flex;
  align-items: flex-start;
}

$bg: #2d3a4b;
$dark_gray: #889aa4;
$light_gray: #eee;
.imgs {
  position: relative;
  height: 36px;
  cursor: pointer;

  img {
    height: 100%;
  }

  span {
    width: 84px;
    line-height: 36px;
    display: inline-block;
    background: rgba(0, 0, 0, 0.4);
    position: absolute;
    left: 0;
    color: #fff;
  }
}

.login-form {
  box-sizing: border-box;
  border-radius: 8px;
  overflow: hidden;
}

.tips {
  font-size: 14px;
  color: #fff;
  margin-bottom: 10px;

  span {
    &:first-of-type {
      margin-right: 16px;
    }
  }
}

.svg-container {
  padding: 6px 5px 6px 15px;
  color: $dark_gray;
  vertical-align: middle;
  width: 30px;
  display: inline-block;
}

.show-pwd {
  position: absolute;
  right: 10px;
  top: 3px;
  font-size: 16px;
  color: $dark_gray;
  cursor: pointer;
  user-select: none;

  ::v-deepsvg-icon {
    vertical-align: 0.3em;
  }
}

.thirdparty-button {
  position: absolute;
  right: 0;
  bottom: 6px;
}
</style>
